// Generated with Blu Age version 201912
package planets;

import com.netfective.bluage.core.helper.ConfigurationHelper;
import com.netfective.bluage.gapwalk.io.support.FileConfigurationUtils;
import com.netfective.bluage.gapwalk.rt.io.support.FileConfiguration;
import com.netfective.bluage.gapwalk.rt.provider.ProgramContainer;
import com.netfective.bluage.gapwalk.rt.provider.ProgramRegistry;
import com.netfective.bluage.gapwalk.rt.provider.ServiceContainer;
import com.netfective.bluage.gapwalk.rt.provider.ServiceRegistry;
import com.netfective.bluage.gapwalk.rt.shared.AwsClientSecretsListener;
import java.util.Map;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.SpringBootConfiguration;
import org.springframework.boot.builder.SpringApplicationBuilder;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.support.SpringBootServletInitializer;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.event.EventListener;
import org.springframework.web.WebApplicationInitializer;
import org.springframework.web.context.WebApplicationContext;
import planets.program.utils.SpringBootApplicationBanner;
/**
 * SpringBoot web application launcher.
 * Entry point for Tomcat deployment and programs/services/scripts registration.
 */
@SpringBootConfiguration
@ComponentScan(basePackages = {"planets.program", "com.netfective.bluage.core","com.netfective.bluage.gapwalk.io.support"})
@EnableConfigurationProperties
public class SpringBootLauncher extends SpringBootServletInitializer implements WebApplicationInitializer {
	/**
	 * The spring web application context. The parent context of all sub programs.
	 */
	private static ConfigurableApplicationContext applicationContext;

	@Override
	protected SpringApplicationBuilder configure(SpringApplicationBuilder application) {
		return application.banner(new SpringBootApplicationBanner("PlanetsDemo")).profiles("PlanetsDemo").sources(SpringBootLauncher.class).listeners(new AwsClientSecretsListener());
	}

	@Override
	protected WebApplicationContext run(SpringApplication application) {
		applicationContext = application.run();
		return (WebApplicationContext) applicationContext;
	}
	
	/**
	 * Get spring web application context.
	 * @return the spring web application context
	 */
	public static ConfigurableApplicationContext getCac() {
		return applicationContext;
	}
	
	/**
	 * Setup default file configurations.
	 * @param configurationHelper the configuration helper
	 * @return file configurations
	 */
	@Bean
	public Map<String, FileConfiguration> fileConfigurations(ConfigurationHelper configurationHelper) {
		return new FileConfigurationUtils().createFileConfigurationFromMap(configurationHelper.getFilesConfiguration()).getFileConfigurations();
	}
	
	
	
	/**
	 * Initialization method called when the spring application is ready.
	 * Register all programs and services to the gapwalk shared context.
	 * @param event the application ready event
	 */
	@EventListener
	public void initialize(ApplicationReadyEvent event) {
		Map<String, ProgramContainer> programContainers = event.getApplicationContext().getBeansOfType(ProgramContainer.class);
		programContainers.values().forEach(ProgramRegistry::registerProgram);
		Map<String, ServiceContainer> serviceContainers = event.getApplicationContext().getBeansOfType(ServiceContainer.class);
		serviceContainers.values().forEach(ServiceRegistry::registerService);
	}
	
}
