// Generated with Blu Age version 201912

package planets.planetsinq.service.impl;

import com.netfective.bluage.gapwalk.datasimplifier.utils.DataUtils;
import com.netfective.bluage.gapwalk.datasimplifier.utils.NumberUtils;
import com.netfective.bluage.gapwalk.datasimplifier.utils.StringUtils;
import com.netfective.bluage.gapwalk.rt.call.ExecutionController;
import com.netfective.bluage.gapwalk.rt.jics.bms.ReceiveMapBuilder;
import com.netfective.bluage.gapwalk.rt.jics.bms.SendMapBuilder;
import com.netfective.bluage.gapwalk.rt.jics.control.ReturnBuilder;
import com.netfective.bluage.gapwalk.rt.utils.ArgUtils;
import com.netfective.bluage.gapwalk.runtime.statements.StringConcatenationBuilder;
import java.util.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import planets.planetsinq.business.context.PlanetsinqContext;
import planets.planetsinq.service.PlanetsinqProcess;

/**
 * Class PlanetsinqProcessImpl
 * 
 * Defines application services for PlanetsinqProcess
 * @see PlanetsinqProcess
 */
@Service("planets.planetsinq.service.PlanetsinqProcess")
public class PlanetsinqProcessImpl implements PlanetsinqProcess {

	/**
	 * The logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(PlanetsinqProcessImpl.class);


	/**
	 * Process operation checkCommarea.
	 * 
	 * PROGRAM-ID.PLANETSINQ.
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void checkCommarea(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		checkCommarea1(ctx, ctrl);
		ReturnBuilder.newInstance(ctx.getDfheiblk(), ctx)
		.withTransID("PINQ")
		.withCommarea(ctx.getCommunicationArea().getCommunicationAreaReference())
		.execute()
		.handleException();
	}

	/**
	 * Process operation checkCommarea1.
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void checkCommarea1(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		ctx.getDfheiblk().bind(ArgUtils.get(ctx, 0));
		ctx.getDfhcommarea().bind(ArgUtils.get(ctx, 1));
		
		/* 
		This program is to be associated with the PINQ CICS transaction */
		if (ctx.getDfheiblk().getEibcalen() == 0) {
			
			/* 
			First access, just send the map */
			sendInitMap(ctx, ctrl);
			return;
		} else {
			getInput(ctx, ctrl);
		}
	}

	/**
	 * Process operation getInput.
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void getInput(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		ReceiveMapBuilder.newInstance(ctx.getDfheiblk(), ctx)
		.withMap("PLNMAP1")
		.withMapset("PLNSET1")
		.into(ctx.getGroup1().getPlanetsmap1iReference())
		.execute()
		.handleException();
		
		/* 
		Basic sanity check */
		if (ctx.getGroup1().getPlntNml() == 0 || DataUtils.isBlank(ctx.getGroup1().getPlntNmiReference())) {
			ctx.getGroup1().setMessageo("Please enter a planet name.");
			sendMapData(ctx, ctrl);
			return;
		} else {
			ctx.getVars().setSanitized(StringUtils.upperCase(ctx.getGroup1().getPlntNmi()));
			
			/* 
			Search in planets data */
			searchPlanet(ctx, ctrl);
			
			/* 
			Send selected planet data (if found) */
			DataUtils.setToBlank(ctx.getGroup1().getMessageoReference());
			if (DataUtils.compare(ctx.getVars().getFoundReference(), "N") == 0) {
				StringConcatenationBuilder.newInstance(ctx.getGroup1().getMessageoReference())
					.addDelimitedByLiteral("'"," ")
					.addDelimitedByLiteral(ctx.getVars().getSanitized()," ")
					.addDelimitedBySize("'")
					.addDelimitedBySize(" is not a planet")
					.end();
			} else {
				ctx.getGroup1().setPlntNmo(ctx.getVars().getSanitized());
				ctx.getFormatted().getFMassReference().setValue(ctx.getPlanetsData().getItemFromMass(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().setFDiamt(ctx.getPlanetsData().getItemFromDiameter(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().setFDens(ctx.getPlanetsData().getItemFromDensity(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().getFDaylReference().setValue(ctx.getPlanetsData().getItemFromDayLength(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().getFSundtReference().setValue(ctx.getPlanetsData().getItemFromSunDistance(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().getFOrbitprReference().setValue(ctx.getPlanetsData().getItemFromOrbitalPeriod(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().getFMeantmpReference().setValue(ctx.getPlanetsData().getItemFromMeanTemperature(ctx.getVars().getIdx() - 1));
				ctx.getFormatted().setFMoonsnb(ctx.getPlanetsData().getItemFromMoonsNb(ctx.getVars().getIdx() - 1));
				ctx.getGroup1().setMasso(ctx.getFormatted().getFMass());
				ctx.getGroup1().setDiamto(ctx.getFormatted().getFDiamtReference().getValue(String.class));
				ctx.getGroup1().setDenso(ctx.getFormatted().getFDensReference().getValue(String.class));
				ctx.getGroup1().setDaylo(ctx.getFormatted().getFDayl());
				ctx.getGroup1().setSundto(ctx.getFormatted().getFSundt());
				ctx.getGroup1().setOrbitpro(ctx.getFormatted().getFOrbitpr());
				ctx.getGroup1().setMeantmpo(ctx.getFormatted().getFMeantmp());
				ctx.getGroup1().setMoonsnbo(ctx.getFormatted().getFMoonsnbReference().getValue(String.class));
				ctx.getGroup1().setHasrngso(ctx.getPlanetsData().getItemFromHasRings(ctx.getVars().getIdx() - 1));
			}
			sendMapData(ctx, ctrl);
		}
	}

	/**
	 * Process operation sendInitMap.
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void sendInitMap(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		SendMapBuilder.newInstance(ctx.getDfheiblk(), ctx)
		.withMap("PLNMAP1")
		.withMapset("PLNSET1")
		.withErase()
		.withNoData()
		.execute()
		.handleException();
	}

	/**
	 * Process operation sendMapData.
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void sendMapData(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		SendMapBuilder.newInstance(ctx.getDfheiblk(), ctx)
		.withMap("PLNMAP1")
		.withMapset("PLNSET1")
		.withData(ctx.getGroup1().getPlanetsmap1oReference())
		.withDataOnly()
		.execute()
		.handleException();
	}

	/**
	 * Process operation searchPlanet.
	 * 
	 *  Search for a planet in the PLANETS data table
	 *  Input: SANITIZED (name to search for)
	 *  Output: FOUND (Y or N)
	 *  Output: IDX (if FOUND is 'Y')
	 * 
	 * @param ctx 
	 * @param ctrl 
	 */
	@Override
	public void searchPlanet(final PlanetsinqContext ctx, final ExecutionController ctrl) {
		ctx.getVars().setFound("N");
		
		/* 
		No INDEXED BY => SEARCH verb is n/a */
		ctx.getVars().setIdx(1);
		while (NumberUtils.le(ctx.getVars().getIdxReference(), 8)) {
			if (DataUtils.compare(ctx.getPlanetsData().getItemFromNameReference(ctx.getVars().getIdx() - 1), ctx.getVars().getSanitizedReference()) == 0) {
				ctx.getVars().setFound("Y");
				return;
			} else {
				ctx.getVars().setIdx(ctx.getVars().getIdx() + 1);
			}
		}
	}

}
