import { NgModule, SystemJsNgModuleLoader, APP_INITIALIZER} from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { HttpClientModule, HttpClient } from '@angular/common/http';

import { CommonsModule } from './commons.module'
import { AppComponent } from './app.component';
import { TransactionRunnerComponent } from './transaction-runner.component';
import { TermComponent } from './term.component';

import { AppRoutingModule } from './app-routing.module';
import { AppService } from './app.service';
import { ConfigService } from './config-service';
import { AppConfigurationMessage } from './configuration-message.module';

// Provided to dynamically loaded components
import { SharedModule } from './shared.module';
import { TermModule } from './term/term.module';
import { DynamicFieldModule } from './dynamic-field/dynamic-field.module';
import { TermModalModule } from './term-modal/term-modal.module';
import { TermService } from './term/term.service';
import { TestingService } from './term/testing.service';
import { TransactionService } from './term/transaction.service';
import { TableModule } from './table/table.module';
import { ModalModule, ModalComponent } from './modal.component';

import { Plnset1Plnmap1Module } from "./maps/plnset1-plnmap1.module";
import { StandardMessageLineModule } from "./maps/standard-messageline.module";
import { RawtextModule } from './maps/rawtext.module';
import { UserInfoService } from './user-info/user-info.service';
import { UserInfoComponent } from './user-info.component';

import { Plnset1Plnmap1Component } from "./maps/plnset1-plnmap1.component";
import { StandardMessageLineComponent } from "./maps/standard-messageline.component";
import { RawtextComponent } from './maps/rawtext.component';

/** Perform application configuration.
 * The application start will delay until the Promise is completed (avoid race conditions).
 * See https://github.com/angular/angular/issues/9047
  */
export function initializeApplication(http: HttpClient, 
    transactionService: TransactionService, userInfoService: UserInfoService, configService: ConfigService) {
    
    // Get backend URL
    return () => configService.configuration()
        .then((configuration: AppConfigurationMessage) => {
            transactionService.configure(configuration);
            userInfoService.configure(configuration);
            return true;
        });
}

@NgModule({
    imports: [
        CommonsModule,
        BrowserModule,
        HttpClientModule,
        AppRoutingModule,
        SharedModule,
        DynamicFieldModule,
        TableModule,
        TermModule,
        TermModalModule,
        ModalModule,
		Plnset1Plnmap1Module,
        StandardMessageLineModule,
        RawtextModule
    ],
    declarations: [
        AppComponent,
        TermComponent,
        TransactionRunnerComponent,
        UserInfoComponent,
    ],
    bootstrap: [
        AppComponent,
    ],
    entryComponents: [
    	ModalComponent,
		Plnset1Plnmap1Component,
        StandardMessageLineComponent,
        RawtextComponent
    ],
    providers: [
        SystemJsNgModuleLoader,
        AppService,
        TermService,
        TransactionService,
        UserInfoService,
        ConfigService,
		TestingService,
        {
            'provide': APP_INITIALIZER,
            'useFactory': initializeApplication,
            'deps': [HttpClient, TransactionService, UserInfoService, ConfigService],
            'multi': true,
        }
    ]
})
export class AppModule { }
