import { Injectable, OnInit, Self } from '@angular/core';

/**
 * Regroups functions to handle insert mode when user press insert key
 */
@Injectable()
export class AppService {
    /* FIELDS ============================================================== */
    private _insertMode = false;
    private _insertModeChanged = new Array<( insertMode: boolean ) => void>();

    /* CONSTRUCTORS ======================================================== */
    constructor() {
        this._initInsertMode();
        window.addEventListener( 'keydown', this.onKeyDown.bind( this ) );
    }

    /* METHODS ============================================================= */
    /* Events -------------------------------------------------------------- */
    private onKeyDown( event: KeyboardEvent ) {
        if( event.key === 'Insert' ) {
            this._updateInsertMode();
        }
    }

    /* Cookie -------------------------------------------------------------- */
    public getCookie( name: string ): string {
        let cookies = this._readCookies();
        return cookies[ name ] || '';
    }

    public setCookie( name: string, value: string ): void {
        let cookies = this._readCookies();
        cookies[ name ] = value || '';
        this._writeCookies( cookies );
    }

    private _readCookies(): any {
        let cookies = {};

        for( let cookieStr of decodeURIComponent( document.cookie ).split( ';' ) ) {
            let index = cookieStr.indexOf( '=' );
            if( index > 0 ) {
                cookies[ cookieStr.substring( 0, index ) ] = cookieStr.substring( index + 1 );
            } else {
                cookies[ cookieStr ] = '';
            }
        }

        return cookies;
    }

    private _writeCookies( cookies: any ): void {
        let cookiesStr = '';

        const SEPARATOR = '; ';
        for( let name of cookies ) {
            cookiesStr += name + '=' + cookies[ name ] + SEPARATOR
        }
        if( cookiesStr.length > 0 ) { // remove last separator
            cookiesStr = cookiesStr.substring( 0, cookiesStr.length - SEPARATOR.length);
        }

        document.cookie = cookiesStr;
    }

    /* Insert mode --------------------------------------------------------- */
    public get insertMode(): boolean {
        return this._insertMode;
    }
    
    // Not be used in specific component
    public set insertMode( insertMode: boolean ) {
    	if( insertMode !== this._insertMode ) {
    		this._insertMode = insertMode;
    		this._insertModeChanged.forEach( f => f( this._insertMode ) );
    	}
    }

    public registerOnInsertModeChange( fn: ( insertMode: boolean ) => void ): void {
        this._insertModeChanged.push( fn );
    }

    private _initInsertMode() {
        this._insertMode = this.getCookie( 'insertMode' ) === 'true';
        console.log( 'Insert mode ' + (this._insertMode ? 'is active' : 'unactivated') );
    }

    private _updateInsertMode() {
        this._insertMode = !this._insertMode;
        this.setCookie( 'insertMode', this._insertMode.toString() );
        console.log( 'Insert mode ' + (this._insertMode ? 'is active' : 'unactivated') );
        this._insertModeChanged.forEach( f => f( this._insertMode ) );
    }
}