import {Component, OnInit, ContentChild, Input, TemplateRef, HostListener} from '@angular/core';
import { Data } from '../term/term.model';

enum More {
    More = "More...", Plus = "+"
}

enum Bottom {
    Bottom = "Bottom", Empty = ""
}

@Component({
    moduleId: module.id,
    selector: 'app-table',
    templateUrl: './table.component.html'
})

export class TableComponent {
    @ContentChild(TemplateRef) templateRef: TemplateRef<any>;

    @Input() rows: any[];
    @Input() nbLinesByRow: number;
    @Input() nbDisplayedEntries: number;
    @Input() startLineNumber: number;

    // End of array
    private end: boolean;
    private bottom: boolean;

    // If valued to "MORE" used pair More/Bottom
    // else used pair +/[empty]
    private pageDown: String;

    // Index from which display array items
    private recordNumber: number = 0;
    private top: true;

    private labelMore:More  = More.More;
    private labelBottom:Bottom = Bottom.Bottom;
    
    // Display table or not
    public display: true;
    
    ngOnInit(): void {

        if (this.rows.length > 0) {
            this.pageDown = this.rows['attributes'].pageDown;
            this.end = this.rows['attributes'].end;
            this.recordNumber = this.rows['attributes'].recordNumber;
            if (!this.recordNumber) {
            	this.recordNumber = 0;
            }
            this.top = this.rows['attributes'].top;
            this.display = this.rows['attributes'].display;
            this.changeLabels();
        }
    }

	public keyEvent(event: KeyboardEvent) { 
        if(event.key === 'PageDown') {
            this.recordNumber = this.recordNumber + this.nbDisplayedEntries;
            if (this.recordNumber > this.rows.length - 1) {
                this.recordNumber = this.rows.length - 1;
            }
		} else if(event.key === 'PageUp') {
            this.recordNumber = this.recordNumber - this.nbDisplayedEntries;
            if (this.recordNumber < 0) {
                this.recordNumber = 0;
            }
        }
        this.bottom = this.recordNumber + this.nbDisplayedEntries == this.rows.length + 1;
        this.end = this.recordNumber == this.rows.length - 1;
        return this.end;
	}

    public setRecordNumber(number: number) { 
        this.recordNumber = number;
	}
	
    changeLabels(): void {

        if (this.pageDown == "MORE" ) {
            this.labelMore  = More.More;
            this.labelBottom  = Bottom.Bottom;
        } else {
            this.labelMore  = More.Plus;
            this.labelBottom  = Bottom.Empty;
        }
    }

    getEndLabel(): String {
        if (this.bottom) {
            return this.labelBottom;
        } else {
            return this.labelMore;
        }
    }

    // Compute class to apply for indicator label
    positionLabel(): String {

        let classes = "";

        if (this.end) {
            if ( this.labelBottom == Bottom.Bottom) {
                classes += "lgo_74 ";
                classes += "lgr_06 ";
            } else {
                // Nothing, disable by ngIf
                return "";
            }
        } else {
            if ( this.labelMore == More.More) {
                classes += "lgo_73 ";
                classes += "lgr_07 ";
            } else {
                classes += "lgo_78 ";
                classes += "lgr_02 ";
            }
        }

        classes += "white";

        return classes;
    }

    // Return true if i th element must be displayed
    displayed(index: number): boolean {
        let display = false;
        
        if (this.top) {
            let nbFirst :number = this.recordNumber - 1;
            let nbLast :number = nbFirst + this.nbDisplayedEntries - 1;

            if (index >= nbFirst && index <= nbLast) {
                display = true;
            }
        } else {

            if (this.nbDisplayedEntries != 0) {
                let pageNbBeforeItem : number = this.recordNumber / this.nbDisplayedEntries;
                let nbFirst :number = parseInt(String(pageNbBeforeItem)) * this.nbDisplayedEntries;
                let nbLast :number = nbFirst + this.nbDisplayedEntries - 1;

                if (index >= nbFirst && index <= nbLast) {
                    display = true;
                }
            } else {
                console.log("Cannot display array items because number of items by page is zero");
            }
        }

        return display;
    }

    // Compute style to apply in order to put "+" indicator label on last displayed line
    overlayLastLine() {

        let style = "{";

        if (this.labelMore == More.Plus) {

            // lrg height css
            let height = 1.4;
            let top = height * ((this.nbDisplayedEntries * this.nbLinesByRow) - 1);

            style += '"position":"absolute"';
            style += ',"top": "' + top + 'em"';

        }
        style += "}";

        return JSON.parse(style);
    }




}

